#!/bin/sh
#
# Warn if a user has too many threads (LWPs)
# ARG1: thread limit
# ARG2: process limit
#
# by Michael Tosch
#
# returns only OK or WARNING

set -f
export PATH
PATH=/bin:/usr/bin:/usr/sbin:/sbin

limit=${1:-2850}
plimit=${2:-850}

case $limit$plimit in
*[!0-9]*)
  echo "
Usage: $0 [WARN threads] [WARN processes]"
  exit 3
  ;;
esac

# Solaris zone awareness
if [ "`{ /sbin/zonename; } 2>/dev/null`" = "global" ]
then
  scope="-z global"
else
  scope="-e"
fi

check=`ps $scope -L -o uid= -o pid= | awk '
{++n[$1]}
p[$2]++==0 {++o[$1]}
END {
 for (u in n) {
  if (n[u]>max) max=n[uid=u]
  if (o[u]>pmax) pmax=o[puid=u]
 }
 if (max>'$limit') {print "uid="uid,"max="max}
 else if (pmax>'$plimit') {print "uid="puid,"pmax="pmax}
}
'`
if [ -z "$check" ]
then
  echo "OK: all users below $limit threads"
  exit 0
else
  eval $check
  user=`getent passwd $uid | cut -f1 -d:`
  if [ -n "$max" ]; then
    echo "WARNING: user $user ($uid) runs $max threads"
  elif [ -n "$pmax" ]; then
    echo "WARNING: user $user ($uid) runs $pmax processes"
  fi
  exit 1
fi
